package com.faraphel.tasks_valider.connectivity.task

import android.util.Log
import com.faraphel.tasks_valider.database.TaskDatabase
import com.faraphel.tasks_valider.database.entities.Task
import com.google.gson.Gson
import fi.iki.elonen.NanoHTTPD


/**
 * A server to handle the task API to allow clients to interact with the database.
 * @param port the port of the server
 * @param database the database to interact with
 */
class TaskServer(
    private val port: Int,
    private val database: TaskDatabase
) : NanoHTTPD(port) {
    private val jsonParser = Gson()

    override fun serve(session: IHTTPSession): Response {
        val method: Method = session.method
        val uri: String = session.uri

        // remove the first slash
        val daoName: String = uri.substring(1)

        // handle the request
        when (method) {
            // get the data from the database
            Method.GET -> {
                return newFixedLengthResponse(
                    Response.Status.OK,
                    "application/json",
                    jsonParser.toJson( database.taskDao().getAll() )
                )
            }
            // insert the data into the database
            Method.POST -> {
                val task = jsonParser.fromJson(
                    session.inputStream.bufferedReader(),
                    Task::class.java
                )
                database.taskDao().insert(task)

                return newFixedLengthResponse(
                    Response.Status.CREATED,
                    "application/json",
                    jsonParser.toJson(task)
                )
            }
            // other methods are not allowed
            else -> {
                return newFixedLengthResponse(
                    Response.Status.METHOD_NOT_ALLOWED,
                    "text/plain",
                    "Method not allowed"
                )
            }
            // TODO(Faraphel): implement a permission system
        }
    }

    /**
     * Start the server with the default configuration
     */
    override fun start() {
        super.start(SOCKET_READ_TIMEOUT, false)
    }
}
