package com.faraphel.tasks_valider.database

import androidx.room.Database
import androidx.room.RoomDatabase
import androidx.room.TypeConverters
import com.faraphel.tasks_valider.database.converters.InstantConverter
import com.faraphel.tasks_valider.database.dao.GroupDao
import com.faraphel.tasks_valider.database.dao.GroupStudentDao
import com.faraphel.tasks_valider.database.dao.StudentDao
import com.faraphel.tasks_valider.database.dao.TaskDao
import com.faraphel.tasks_valider.database.dao.TaskGroupDao
import com.faraphel.tasks_valider.database.dao.TeacherDao
import com.faraphel.tasks_valider.database.dao.base.BaseDao
import com.faraphel.tasks_valider.database.entities.Group
import com.faraphel.tasks_valider.database.entities.GroupStudent
import com.faraphel.tasks_valider.database.entities.Student
import com.faraphel.tasks_valider.database.entities.Task
import com.faraphel.tasks_valider.database.entities.TaskGroup
import com.faraphel.tasks_valider.database.entities.Teacher


/**
 * The database for the tasks application.
 * Contains the entities and the relations between them.
 */
@Database(
    entities = [
        Group::class,
        Student::class,
        Teacher::class,
        Task::class,

        GroupStudent::class,
        TaskGroup::class,
    ],
    version = 1
)
@TypeConverters(
    InstantConverter::class
)
abstract class TaskDatabase : RoomDatabase() {
    // entities
    abstract fun groupDao(): GroupDao
    abstract fun studentDao(): StudentDao
    abstract fun teacherDao(): TeacherDao
    abstract fun taskDao(): TaskDao

    // relations
    abstract fun groupStudentDao(): GroupStudentDao
    abstract fun taskGroupDao(): TaskGroupDao

    /**
     * Get the DAO from the name of the dao.
     */
    @Suppress("UNCHECKED_CAST")
    fun <Entity> daoFromName(name: String): BaseDao<Entity>? {
        return when (name) {
            "group" -> groupDao()
            "student" -> studentDao()
            "teacher" -> teacherDao()
            "task" -> taskDao()
            "group_student" -> groupStudentDao()
            "task_group" -> taskGroupDao()
            else -> null
        } as BaseDao<Entity>?
    }
}
